﻿param(
[string]$PLAIN_TEXT
)


try{
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Definitions.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.dll
    Add-Type -Path $PSScriptRoot\System.Collections.Immutable.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.Number.dll
    Add-Type -Path $PSScriptRoot\Microsoft.Recognizers.Text.NumberWithUnit.dll
    Add-Type -Path $PSScriptRoot\UnitsNet.dll
}
catch{
    $error[0].Exception.GetBaseException().LoaderExceptions
}


$result = [Microsoft.Recognizers.Text.NumberWithUnit.NumberWithUnitRecognizer]::RecognizeDimension($PLAIN_TEXT, "en")


if($result.Count -eq 0){
$result = [Microsoft.Recognizers.Text.NumberWithUnit.NumberWithUnitRecognizer]::RecognizeTemperature($PLAIN_TEXT,"en")
}

if($result.Count -eq 0){
$result = [Microsoft.Recognizers.Text.NumberWithUnit.NumberWithUnitRecognizer]::RecognizeAge($PLAIN_TEXT,"en")
    
}



if($result.Count -eq 0){

Write-Error "Text not recognized"
return
}



$unit = $result[0].Resolution["unit"]
[double]$value = $result[0].Resolution["value"]


$output = "Unit: " + $unit + ", Value: " + "$value" + "`r`n"



$baseVal_mass = New-Object UnitsNet.Mass

switch ($unit) {
   "Kilogram" {$baseVal_mass = [UnitsNet.Mass]::FromKilograms($value); break}
   "Gram" {$baseVal_mass = [UnitsNet.Mass]::FromGrams($value); break}
   "Milligram" {$baseVal_mass = [UnitsNet.Mass]::FromMilligrams($value); break}
   "Ton" {$baseVal_mass = [UnitsNet.Mass]::FromTonnes($value); break}
   "Pound" {$baseVal_mass = [UnitsNet.Mass]::FromPounds($value); break}
   "Pound" {$baseVal_mass = [UnitsNet.Mass]::FromOunces($value); break}
   default {$baseVal_mass = $null; break}
}


if($baseVal_mass -ne $null){
    $output = $output+ [string]$baseVal_mass.Kilograms + " kg `r`n"
    $output = $output + [string]$baseVal_mass.Grams + " g`r`n"
    $output = $output + [string]$baseVal_mass.Milligrams + " mg`r`n"
    $output = $output + [string]$baseVal_mass.Tonnes + " ton(s)`r`n"
    $output = $output + [string]$baseVal_mass.Pounds + " lbs`r`n"
    $output = $output + [string]$baseVal_mass.Ounces + " oz"

    Write-Output($output)

    return
}



$baseVal_volume = New-Object UnitsNet.Volume


switch ($unit) {
   "Cubic meter" {$baseVal_volume = [UnitsNet.Volume]::FromCubicMeters($value); break}
   "Cubic centimeter" {$baseVal_volume = [UnitsNet.Volume]::FromCubicCentimeters($value); break}
   "Cubic millimiter" {$baseVal_volume = [UnitsNet.Volume]::FromCubicMillimeters($value); break}
   "Liter" {$baseVal_volume = [UnitsNet.Volume]::FromLiters($value); break}
   "Centiliter" {$baseVal_volume = [UnitsNet.Volume]::FromCentiliters($value); break}
   "Deciliter" {$baseVal_volume = [UnitsNet.Volume]::FromDeciliters($value); break}
   "Milliliter" {$baseVal_volume = [UnitsNet.Volume]::FromMilliliters($value); break}
   "Cubic yard" {$baseVal_volume = [UnitsNet.Volume]::FromCubicYards($value); break}
   "Cubic inch" {$baseVal_volume = [UnitsNet.Volume]::FromCubicInches($value); break}
   "Cubic foot" {$baseVal_volume = [UnitsNet.Volume]::FromCubicFeet($value); break}  
   "Cubic mile" {$baseVal_volume = [UnitsNet.Volume]::FromCubicMiles($value); break}
   default {$baseVal_volume = $null; break}
}


if($baseVal_volume -ne $null){

    $output = $output + [string]$baseVal_volume.CubicMeters + " m³`r`n"
    $output = $output + [string]$baseVal_volume.CubicCentimeters + " cm³`r`n"
    $output = $output + [string]$baseVal_volume.CubicMillimeters + " mm³`r`n"
    $output = $output + [string]$baseVal_volume.Liters + " l`r`n"
    $output = $output + [string]$baseVal_volume.Centiliters + " cl`r`n"
    $output = $output + [string]$baseVal_volume.Deciliters + " dl`r`n"
    $output = $output + [string]$baseVal_volume.Milliliters + " ml`r`n"
    $output = $output + [string]$baseVal_volume.CubicYards + " yd³`r`n"
    $output = $output + [string]$baseVal_volume.CubicInches + " in³`r`n"
    $output = $output + [string]$baseVal_volume.CubicFeet + " ft³`r`n"
    $output = $output + [string]$baseVal_volume.CubicMiles + " mi³"
    Write-Output $output

}


$baseVal_temp = New-Object UnitsNet.Temperature


switch ($unit) {
   "F" {$baseVal_temp = [UnitsNet.Temperature]::FromDegreesFahrenheit($value); break}
   "K" {$baseVal_temp = [UnitsNet.Temperature]::FromKelvins($value); break}
   "R" {$baseVal_temp = [UnitsNet.Temperature]::FromDegreesRankine($value); break}
   "D" {$baseVal_temp = [UnitsNet.Temperature]::FromDegreesDelisle($value); break}
   "C" {$baseVal_temp = [UnitsNet.Temperature]::FromDegreesCelsius($value); break}
   default {$baseVal_temp = $null; break}
}

if($baseVal_temp -ne $null){
    $output = $output + [string]$baseVal_temp.DegreesFahrenheit + " °F`r`n"
    $output = $output + [string]$baseVal_temp.Kelvins + " K`r`n"
    $output = $output + [string]$baseVal_temp.DegreesRankine + " °R`r`n"
    $output = $output + [string]$baseVal_temp.DegreesDelisle + " °D`r`n"
    $output = $output + [string]$baseVal_temp.DegreesCelsius + " °C"
    Write-Output $output

}


$baseVal_speed = New-Object UnitsNet.Speed

switch ($unit){
   "Meter per second" {$baseVal_speed = [UnitsNet.Speed]::FromMetersPerSecond($value); break}
   "Kilometer per hour" {$baseVal_speed = [UnitsNet.Speed]::FromKilometersPerHour($value); break}
   "Kilometer per minute" {$baseVal_speed = [UnitsNet.Speed]::FromKilometersPerMinutes($value); break}
    "Kilometer per second" {$baseVal_speed = [UnitsNet.Speed]::FromKilometersPerSecond($value); break}
    "Mile per hour" {$baseVal_speed = [UnitsNet.Speed]::FromMilesPerHour($value); break}
    "Knot" {$baseVal_speed = [UnitsNet.Speed]::FromKnots($value); break}
    "Foot per second" {$baseVal_speed = [UnitsNet.Speed]::FromFeetPerSecond($value); break}
    "Foot per minute" {$baseVal_speed = [UnitsNet.Speed]::FromFeetPerMinute($value); break}
    "Yard per minute" {$baseVal_speed = [UnitsNet.Speed]::FromYardsPerMinute($value); break}
    "Yard per second" {$baseVal_speed = [UnitsNet.Speed]::FromYardsPerSecond($value); break}
   default {$baseVal_speed = $null; break}

}

if($baseVal_speed -ne $null){
    $output =  $output + [string]$baseVal_speed.MetersPerSecond + " m/s`r`n"
    $output =  $output + [string]$baseVal_speed.KilometersPerHour + " km/h`r`n"
    $output =  $output + [string]$baseVal_speed.KilometersPerMinutes + " km/m`r`n"
    $output =  $output + [string]$baseVal_speed.KilometersPerSecond + " km/s`r`n"
    $output =  $output + [string]$baseVal_speed.MilesPerHour + " mph`r`n"
    $output =  $output + [string]$baseVal_speed.Knots + " knot/h`r`n"
    $output =  $output + [string]$baseVal_speed.FeetPerSecond + " ft/s`r`n"
    $output =  $output + [string]$baseVal_speed.FeetPerMinute + " ft/min`r`n"
    $output =  $output + [string]$baseVal_speed.YardsPerMinute + " yard/min`r`n"
    $output =  $output + [string]$baseVal_speed.YardsPerSecond + " yard/s"
    Write-Output $output

}


$baseVal_length = New-Object UnitsNet.Length


switch ($unit){
   "Kilometer" {$baseVal_length = [UnitsNet.Length]::FromKilometers($value); break}
   "Meter" {$baseVal_length = [UnitsNet.Length]::FromMeters($value); break}
   "Decimeter" {$baseVal_length = [UnitsNet.Length]::FromDecimeters($value); break}
   "Centimeter" {$baseVal_length = [UnitsNet.Length]::FromCentimeters($value); break}
   "Millimeter" {$baseVal_length = [UnitsNet.Length]::FromMillimeters($value); break}
   "Micrometer" {$baseVal_length = [UnitsNet.Length]::FromMicrometers($value); break}
   "Nanometer" {$baseVal_length = [UnitsNet.Length]::FromNanometers($value); break}
   "Mile" {$baseVal_length = [UnitsNet.Length]::FromMiles($value); break}
   "Yard" {$baseVal_length = [UnitsNet.Length]::FromYards($value); break}
   "Inch" {$baseVal_length = [UnitsNet.Length]::FromInches($value); break}
   "Foot" {$baseVal_length = [UnitsNet.Length]::FromFeet($value); break}
   "Pt" {$baseVal_length = [UnitsNet.Length]::FromDtpPoints($value); break}
   default {$baseVal_length = $null; break}

}

if($baseVal_length -ne $null){
    $output =  $output + [string]$baseVal_length.Kilometers + " km`r`n"
    $output =  $output + [string]$baseVal_length.Meters + " m`r`n"
    $output =  $output + [string]$baseVal_length.Decimeters + " dm`r`n"
    $output =  $output + [string]$baseVal_length.Centimeters + " cm`r`n"
    $output =  $output + [string]$baseVal_length.Millimeters + " mm`r`n"
    $output =  $output + [string]$baseVal_length.Nanometers + " nm`r`n"
    $output =  $output + [string]$baseVal_length.Miles + " miles`r`n"
    $output =  $output + [string]$baseVal_length.Yards + " yards`r`n"
    $output =  $output + [string]$baseVal_length.Inches + " in`r`n"
    $output =  $output + [string]$baseVal_length.Feet + " ft`r`n"
    $output =  $output + [string]$baseVal_length.DtpPoints + " dtp"
    Write-Output $output

}


$baseVal_info = New-Object UnitsNet.Information

switch ($unit){
   "Bit" {$baseVal_info = [UnitsNet.Information]::FromBits($value); break}
   "Kilobit" {$baseVal_info = [UnitsNet.Information]::FromKilobits($value); break}
   "Megabit" {$baseVal_info = [UnitsNet.Information]::FromMegabits($value); break}
   "Gigabit" {$baseVal_info = [UnitsNet.Information]::FromGigabits($value); break}
   "Terabit" {$baseVal_info = [UnitsNet.Information]::FromTerabits($value); break}
   "Petabit" {$baseVal_info = [UnitsNet.Information]::FromPetabits($value); break}
   "Byte" {$baseVal_info = [UnitsNet.Information]::FromBytes($value); break}
   "Kilobyte" {$baseVal_info = [UnitsNet.Information]::FromKilobytes($value); break}
   "Megabyte" {$baseVal_info = [UnitsNet.Information]::FromMegabytes($value); break}
   "Gigabyte" {$baseVal_info = [UnitsNet.Information]::FromGigabytes($value); break}
   "Terabyte" {$baseVal_info = [UnitsNet.Information]::FromTerabytes($value); break}
   "Petabyte" {$baseVal_info = [UnitsNet.Information]::FromPetabytes($value); break}
   default {$baseVal_info = $null; break}

}

if($baseVal_info -ne $null){
    $output =  $output + [string]$baseVal_info.Bits + " bit`r`n"
    $output =  $output + [string]$baseVal_info.Kilobits + " kbit`r`n"
    $output =  $output + [string]$baseVal_info.Megabits + " mbit`r`n"
    $output =  $output + [string]$baseVal_info.Gigabits + " gbit`r`n"
    $output =  $output + [string]$baseVal_info.Terabits + " tbit`r`n"
    $output =  $output + [string]$baseVal_info.Petabits + " pbit`r`n"
    $output =  $output + [string]$baseVal_info.Bytes + " byte`r`n"
    $output =  $output + [string]$baseVal_info.Kilobytes + " kbyte`r`n"
    $output =  $output + [string]$baseVal_info.Megabytes + " mbyte`r`n"
    $output =  $output + [string]$baseVal_info.Gigabytes + " gbyte`r`n"
    $output =  $output + [string]$baseVal_info.Terabytes + " tbyte`r`n"
    $output =  $output + [string]$baseVal_info.Petabytes + " pbyte"
    Write-Output $output

}


$baseVal_area = New-Object UnitsNet.Area

switch ($unit){
   "Square kilometer" {$baseVal_area = [UnitsNet.Area]::FromSquareKilometers($value); break}
   "Square meter" {$baseVal_area = [UnitsNet.Area]::FromSquareMeters($value); break}
   "Square decimeter" {$baseVal_area = [UnitsNet.Area]::FromSquareDecimeters($value); break}
   "Square centimeter" {$baseVal_area = [UnitsNet.Area]::FromSquareCentimeters($value); break}
   "Square millimeter" {$baseVal_area = [UnitsNet.Area]::FromSquareMillimeters($value); break}
   "Square inch" {$baseVal_area = [UnitsNet.Area]::FromSquareInches($value); break}
   "Square foot" {$baseVal_area = [UnitsNet.Area]::FromSquareFeet($value); break}
   "Square mile" {$baseVal_area = [UnitsNet.Area]::FromSquareMiles($value); break}
   "Square yard" {$baseVal_area = [UnitsNet.Area]::FromSquareYards($value); break}
   "Acre" {$baseVal_area = [UnitsNet.Area]::FromAcres($value); break}
   default {$baseVal_area = $null; break}

}

if($baseVal_area -ne $null){
    $output =  $output + [string]$baseVal_area.SquareKilometers + " km²`r`n"
    $output =  $output + [string]$baseVal_area.SquareMeters + " m²`r`n"
    $output =  $output + [string]$baseVal_area.SquareDecimeters + " dm²`r`n"
    $output =  $output + [string]$baseVal_area.SquareCentimeters + " cm²`r`n"
    $output =  $output + [string]$baseVal_area.SquareMillimeters + " mm²`r`n"
    $output =  $output + [string]$baseVal_area.SquareInches + " in²`r`n"
    $output =  $output + [string]$baseVal_area.SquareFeet + " ft²`r`n"
    $output =  $output + [string]$baseVal_area.SquareMiles + " mi²`r`n"
    $output =  $output + [string]$baseVal_area.SquareYards + " yd²`r`n"
    $output =  $output + [string]$baseVal_area.Acres + " acres"
    Write-Output $output

}


$baseVal_age = New-Object UnitsNet.Duration

switch ($unit){
   "Year" {$baseVal_age = [UnitsNet.Duration]::FromYears365($value); break}
   "Month" {$baseVal_age = [UnitsNet.Duration]::FromMonths30($value); break}
   "Week" {$baseVal_age = [UnitsNet.Duration]::FromWeeks($value); break}
   "Day" {$baseVal_age = [UnitsNet.Duration]::FromDays($value); break}
   default {$baseVal_age = $null; break}

}

if($baseVal_age -ne $null){
    $output =  $output + [string]$baseVal_age.Years365 + " years`r`n"
    $output =  $output + [string]$baseVal_age.Months30 + " months`r`n"
     $output =  $output + [string]$baseVal_age.Weeks + " weeks`r`n"
     $output =  $output + [string]$baseVal_age.Days + " days"
    Write-Output $output

}

<#
https://github.com/angularsen/UnitsNet#dynamic-parsing
https://github.com/Microsoft/Recognizers-Text/blob/199ffbca89d40654554ea4e8bc71238588331f57/.NET/Microsoft.Recognizers.Definitions.Common/English/NumbersWithUnitDefinitions.cs
#>